-- Checkup is intended to check for various problems in a database's schemas ans parameters
-- that need to be addressed as new problems are found they are added to checkup.
-- Checkup.sql also recompiles any invalid objects before it reports invalid objects.
-- Doug Jackson
--

spool checkup.txt

define excl_trig="'''-MONTHLY_HIST_STATMENTTRIGGER,WEEKLY_HIST_STATMENTTRIGGER,ANALOGDB_AVG_INS_WESTEX'''"
define excl_mod1="'''RISK,PWRPLANT,HEAT,EXT_HEAT,ONG_WMRS'''"
define excl_mod2="'''DBO,GISB,ALTRA,RONEOKPR,MEST,VHIST,FCOWNER'''"
define excl_mod_obj="'''PK_QSTAG_ANALYSIS_UPLOAD,QSTAG_ANALYSIS_UPLOAD,QSTAG_VOL_PREDETERMINED,DBA_HIST_PARAMETER'''"
define excl_dba1="'''BRC,DJACKSON,SPARKER,ANDREW_GONCHAROV,JIM_CRAGEN,KIRILL_ZOLOTKO,KONSTANTIN_MILKO,LEONID_MIKHAILOV,LEV_VAYSBERG,OLEG_PYATNYCHKO,RUSLAN_NAUMOV,VLADIMIR_GORODEKIY'''"
define excl_dba2="'''BACKUP,SLEE,MJENKINS,SHEILA_GRAY,CHRISTOPHER_HASKINS,TOPAZ,CMYERS,OEM_USER,SYS,SYSTEM,INTERNAL,CTXSYS,PORTAL,PORTAL_SSO,WMRS,SYSMAN,WKSYS,MDSYS,KEN'''"
define excl_analyze="'''DGIS1,DGIS2,DGIS3,DGIS4,DGIS5,DGIS6,DGIS7,DGIS8,DGIS9,DGIS10,DGIS10,DGIS11,DGIS12,DONGL,IMAG,APANO,DBQES,TBQES,'''"
define excl_all="'''DGIS1,DGIS2,DGIS3,DGIS4,DGIS5,DGIS6,DGIS7,DGIS8,DGIS9,DGIS10,DGIS10,DGIS11,DGIS12,DONGL,DBQES,'''"

SET SERVEROUTPUT ON SIZE 1000000;
SET FEEDBACK OFF;
SET TAB OFF;
SET VERIFY OFF;
SET LINESIZE 132;
SET PAGESIZE 3000;

VARIABLE table_check VARCHAR2(30)
BEGIN
  SELECT max('x') INTO :table_check FROM dba_tables 
    WHERE table_name IN ('TEMP_DB_CHECKUP','TEMP_DB_CHECKUP_WARN')
    and owner = user;
    dbms_utility.exec_ddl_statement ('drop table temp_db_checkup');
    dbms_utility.exec_ddl_statement ('drop table temp_db_checkup_warn');
  EXCEPTION
    WHEN OTHERS THEN
      --dbms_output.put_line(substr(SQLERRM,1,200));
      null;
END;
/

CREATE TABLE temp_db_checkup (setting varchar2(60), value varchar2(40), comments varchar2(200))
  storage( initial 8K next 8k ) tablespace users;
CREATE TABLE temp_db_checkup_warn (warning varchar2(200))
  storage( initial 8K next 8k  ) tablespace users;

--Optional Arguments Passed to checkup.sql are converted to bind variables
var bv_os varchar2(10);
var bv_os_ver varchar2(50);
var bv_storage varchar2(30);
begin
  -- bv_os = AIX LINUX or SUN
  select nvl('&1','n/a') into :bv_os from dual ;
  -- bv_os_ver - Examples: AIX 5300-03, Linux version 2.4.21-15.ELsmp,  SunOS 5.8
  select nvl('&2','n/a') into :bv_os_ver from dual ;
  -- bv_stroage = netapp0X or unknown 
  select nvl('&3','n/a') into :bv_storage from dual ;
end;
/

DECLARE

  CURSOR cur_version is
    select BANNER from v$version where banner like '%Oracle%';
  version varchar2(100);
  
  CURSOR cur_database is
    select NAME, LOG_MODE from v$database;
  database_rec cur_database%ROWTYPE;

  CURSOR cur_sort_stats is
    select a.value disk_sorts, b.value memory_sorts,
    to_char((100*b.value)/decode((a.value+b.value),0,1,(a.value+b.value)),'999999999999.99') pct_mem_sorts
    from v$sysstat a, v$sysstat b 
    where a.name = 'sorts (disk)' and b.name = 'sorts (memory)';
  disk_sorts number;
  memory_sorts number;
  pct_mem_sorts number;
  
  CURSOR cur_free_memory is
    select bytes free_memory from v$sgastat where v$sgastat.name = 'free memory';
  free_memory number;
  
  CURSOR cur_buffer_hit_ratio is
    select (1-(sum(decode(name,'physical reads',value,0))/(sum(decode(name,'db block gets',value,0))+sum(decode(name,'consistent gets',value,0)))))*100 from v$sysstat;
  buffer_hit_ratio number;

  CURSOR cur_buffer_wait_ratio is
    select a.buffer_busy_waits, b.db_block_gets, c.consistent_gets
    from 
      (select total_waits as buffer_busy_waits from v$system_event where event = 'buffer busy waits') a,
      (select value as db_block_gets FROM v$sysstat WHERE name = 'db block gets') b,
      (select value as consistent_gets FROM v$sysstat WHERE name = 'consistent gets') c ;
  rec_buffer_wait_ratio cur_buffer_wait_ratio%ROWTYPE ;
  
  CURSOR cur_dictionary_hit_ratio is
    select TRUNC((1-(sum(getmisses)/sum(gets)))* 100) dictionary_hit_ratio from v$rowcache;
  dictionary_hit_ratio number;
  
  CURSOR cur_library_hit_ratio is
    select sum(pins)/(sum(pins)+sum(reloads))*100 "library_hit_ratio" from v$librarycache;
  library_hit_ratio number;

  CURSOR cur_system_own is
    select distinct owner from dba_tables where owner not in 
	('SYS','SYSTEM','DBSNMP','OUTLN','ORDSYS','MTSSYS','MDSYS','ORDPLUGINS','AURORA$JIS$UTILITY$','OSE$HTTP$ADMIN','AURORA$ORB$UNAUTHENTICATED','WMSYS','MGMT_VIEW')
	and tablespace_name = 'SYSTEM';

  CURSOR cur_system_def is
    select username from dba_users where username not in 
	('SYS','SYSTEM','OUTLN','DIP','ORDSYS','MDSYS','ORDPLUGINS','TRACESVR','DBSNMP','CTXSYS','AURORA$JIS$UTILITY$','OSE$HTTP$ADMIN','AURORA$ORB$UNAUTHENTICATED','WMSYS','TSMSYS','MGMT_VIEW') 
	and default_tablespace = 'SYSTEM';

  CURSOR cur_system_tmp is
    select username from dba_users where username not in 
	('SYS','SYSTEM','OUTLN','ORDSYS','MDSYS','TRACESVR','DBSNMP','CTXSYS','AURORA$JIS$UTILITY$','OSE$HTTP$ADMIN','AURORA$ORB$UNAUTHENTICATED','WMSYS','TSMSYS') 
	and temporary_tablespace = 'SYSTEM';

  CURSOR cur_def_passwd is
    select 'User: '||username||' still has its default password of '||
	  decode(username,'DBSNMP','DBSNMP','OUTLN','OUTLN','SYS','Change_on_install','SYSTEM','Manager')||'.'
	 from dba_users 
	 where (username = 'SYS' and password in 
	 ('D2FE896D41A38613','D4C5016086B2DC6A','5638228DAF52805F','8A8F025737A9097A','4DE42795E66117AE',
          '66BC3FF56063CE97','57D7CFA12BB5BABF','A9A57E819B32A03D','64074AF827F4B74A','41B328CA13F70713',
          '0B4409DDD5688913','6CFF570939041278','3522F32DD32A9706','BE29E31B2B0EDA33','5AC333703DE0DBD4',
          '43BE121A2A135FF3','1FA22316B703EBDD','12CFB5AE1D087BA3','380E3D3AD5CE32D4','2563EFAAE44E785A',
          'E7686462E8CD2F5E','691C5E7E424B821A'))
	 or (username = 'SYSTEM' and password in 
         ('D4DF7931AB130E37','4438308EE0CAFB7F','FAAD7ADAF48B5F45','685657E9DC29E185','49B70B505DF0247F',
          'B49C4279EBD8D1A8','604101D3AACE7E88','02AB2DB93C952A8F','203CD8CF183E716C','135176FFB5BA07C9',
          '8BF0DA8E551DE1B9','1B9F1F9A5CB9EB31','2D594E86F93B17A1','4861C2264FB17936','970BAA5B81930A40',
          'E4519FCD3A565446','66A490AEAA61FF72','10B0C2DA37E11872','D5DD57A09A63AA38','69C27FA786BA774C',
          '86FDB286770CD4B9','B171042374D7E6A2','D7C18B3B3F2A4D4B','D652D8A1303025AF'))
         or ( username = 'DBSNMP' and password = 'E066D214D5421CCC')
         or ( username = 'OUTLN'  and password in ('17CC3815F8C4B590','4A3BA55E08595C81'));
	    
  CURSOR cur_temp_is_perm is
    select tablespace_name from dba_tablespaces
    where contents = 'PERMANENT' and status = 'ONLINE'
    and tablespace_name not in
      (select distinct tablespace_name from dba_segments 
       where segment_type <> 'TEMPORARY' and tablespace_name in 
         (select distinct temporary_tablespace from dba_users))
    and tablespace_name in 
      (select distinct temporary_tablespace from dba_users)
    and tablespace_name in
      (select distinct tablespace_name from dba_segments);

  CURSOR cur_temp_perm is
    select u.username, s.tablespace_name from dba_segments s, dba_users u
    where s.tablespace_name = u.temporary_tablespace
    AND s.segment_type <> 'TEMPORARY' AND tablespace_name IN
    (select distinct temporary_tablespace from dba_users 
     where temporary_tablespace NOT IN ('SYSTEM','DRSYS'))
    group by u.username, s.tablespace_name;

  CURSOR cur_rbseg is
    select tablespace_name, segment_name, pct_increase from dba_rollback_segs where tablespace_name <> 'SYSTEM';
  rbseg_rec cur_rbseg%ROWTYPE;
  last_ts varchar2(30) := 'none';

  CURSOR cur_freebytes (tablespace varchar2) is
    select sum(bytes) freebytes 
      from dba_free_space where tablespace_name = tablespace ;
  lv_freebytes number;

  CURSOR cur_fixedcost (tablespace varchar2) is
    select sum(initial_extent + (next_extent * (min_extents-1))) fixedcostbytes
    from dba_rollback_segs
    where tablespace_name = tablespace ;
  lv_fixedcost number;

  CURSOR cur_failafter (tablespace varchar2, segment varchar2, freebytes number, fixedcost number ) is
    select max_extents, trunc((freebytes - fixedcost)/next_extent) failafter
      from dba_rollback_segs 
      where tablespace_name = tablespace 
      and segment_name = segment ;
  failafter_rec cur_failafter%ROWTYPE;

  CURSOR cur_rbs_online is
    select count(1) "ONLINE", sum(a.xacts) active, sum(a.waits) totalwaits from v$rollstat a, v$rollname b
      where a.usn = b.usn and a.status = 'ONLINE' and b.name not in ('SYSTEM','RBS_BIG');
  rbs_online_rec cur_rbs_online%ROWTYPE; 

  CURSOR cur_rbs_share is 
    select substr(tablespace_name,1,30) tablespace, sum(decode(segment_type,'ROLLBACK',0,1)) err
      from dba_segments 
      where TABLESPACE_NAME in (select distinct tablespace_name from dba_rollback_segs where tablespace_name <> 'SYSTEM') 
      group by tablespace_name;
  rbs_share_rec cur_rbs_share%ROWTYPE;

  CURSOR cur_temp_ts is
    select (b.free/a.avail*100) pct, a.avail 
  	from 
      (select sum(df.bytes/1024/1024) avail from dba_data_files df where df.tablespace_name = 'TEMP') a,
      (select sum(fs.bytes/1024/1024) free  from dba_free_space fs where fs.tablespace_name = 'TEMP') b;
  temp_ts_rec cur_temp_ts%ROWTYPE;
  
  CURSOR cur_backup_ts is
    select tablespace_name from dba_data_files ddf
      where file_id=(select file# from v$backup vb where ddf.file_id=vb.file# and vb.status='ACTIVE');
  backup_ts_rec cur_backup_ts%ROWTYPE;

  CURSOR cur_trans is
    select a.value+b.value total, (to_char(a.value)||' Commited '||to_char(b.value)||' Rolled_Back') comments
    from v$sysstat a, v$sysstat b 
    where a.statistic# = 4 and b.statistic#=5 ;
  trans_rec cur_trans%ROWTYPE;
  
  CURSOR cur_invalid_obj is
    select count(1) cnt, owner, min(object_name) object_name 
    from dba_objects where status = 'INVALID' and object_type not in ('UNDEFINED') and owner <> 'PUBLIC'
    and instr(&excl_all, database_rec.name)=0
    group by owner;
  invalid_obj_rec cur_invalid_obj%ROWTYPE;

  CURSOR cur_cached_cursors is
      select value, used, decode(value, 0, 0, (used / value * 100))  usage
      from ( select max(s.value) used from v$statname n, v$sesstat s
      where n.name = 'session cursor cache count' and s.statistic# = n.statistic# ),
      ( select value from v$parameter where name = 'session_cached_cursors' );
  cached_cursors_rec cur_cached_cursors%ROWTYPE;

  CURSOR cur_open_cursors is
  select value, used,  (used / value * 100) usage from
    ( select max(sum(s.value)) used from v$statname  n, v$sesstat s
      where n.name = 'session cursor cache count'
      and s.statistic# = n.statistic#
      group by s.sid ),
    ( select value from v$parameter where name = 'open_cursors' );
  open_cursors_rec cur_open_cursors%ROWTYPE;

  CURSOR cur_invalid_syn is
    select count(1) cnt, owner, min(synonym_name) synonym_name
      from all_synonyms s 
      where db_link is null 
      and synonym_name <> 'DEF$_SCHEDULE'
      and not exists  
        (select 1 from all_objects o where o.owner = s.table_owner and o.object_name = s.table_name)
      and instr(&excl_all, database_rec.name)=0
      group by owner;
  invalid_syn_rec cur_invalid_syn%ROWTYPE;

  CURSOR cur_jobs is
    select count(1) failed from dba_jobs where (broken='Y' or failures > 0)
    and instr(&excl_all, database_rec.name)=0;
  jobs_rec cur_jobs%ROWTYPE;
  
  CURSOR cur_part_ind is
  select count(1) cnt from dba_ind_partitions where status = 'UNUSABLE';
  part_ind_rec cur_part_ind%ROWTYPE;
  
  CURSOR cur_modified is
  select count(1) cnt 
    from dba_objects o, v$instance i 
    where i.host_name not in ('rsprod8','rstest7')
    and o.last_ddl_time >= sysdate-1 
    and to_char(o.last_ddl_time,'HH24MM') < 1800 
    and to_char(o.last_ddl_time,'HH24MM') > 0800
    and o.owner not in ('SYSTEM','SYS','PUBLIC','DJACKSON','SLEE','BRC')
    and instr(&excl_mod1,o.owner)=0 
    and instr(&excl_mod2,o.owner)=0
    and instr(&excl_mod_obj,o.object_name)=0;
  modified_rec cur_modified%ROWTYPE;
  
  CURSOR cur_dba is
  select username as username from v$pwfile_users
    where ( instr(&excl_dba1,username)=0 and instr(&excl_dba2,username)=0 )
  union
  select grantee as username from dba_role_privs where granted_role = 'DBA'
    and ( instr(&excl_dba1,grantee)=0 and instr(&excl_dba2,grantee)=0 ) ;
  dba_username  varchar2(80);
  dba_list varchar2(80);

  -- Variables from v$parameter
  db_block_buffers number;
  db_cache_size number;
  log_buffer number;
  db_block_size number;
  -- db_file_multiblock_read_count
  db_multiblock number;
  shared_pool_size number;
  large_pool_size number;
  java_pool_size number;
  pga_aggregate_target number;
  sga_target varchar2(20);
  sga_max_size varchar2(20);
  filesystemio_options varchar2(20);
  disk_asynch_io varchar2(20);
  workarea_size_policy varchar2(12);
  l_pga_hit number;
  l_pga_suggested_mg number;
  sort_area_size number;
  sort_area_retained_size number;
  opt_mode varchar2(20);
  -- Not avail in version 7
  analyzed_oldest date;
  analyzed_newest date;
  rbs_mode varchar2(10);
  NoSpaceErrCnt varchar2(6);
  BeginNoSpace varchar2(12);
  db_version varchar2(5);
  -- Shared Pool Reserver Size Calculation
  l_shared_pool_mod varchar2(20);
  l_free_space number;
  l_avg_free_size number;
  l_used_space number;
  l_avg_used_size number;
  l_request_failures number;
  l_last_failure_size number;
  l_shared_pool_reserved_size number;
  l_host_name varchar2(64);
  l_db_cache_size_advice number;
  lv_flag char(3) := 'NO';
  lv_comment varchar2(120) := '' ;
  lv_owner varchar2(30) := '' ;
  lv_username varchar2(30) := '' ;
  lv_tsname varchar2(30) := '' ;
  lv_value1 varchar2(30) := '';
  lv_value2 varchar2(20) := '';
  lv_cmd varchar2(2000) := '';
    
BEGIN

  -- Exceptions allow for Oracle versions (7.x) where the variables do not exist
  BEGIN
    select decode(min(nvl(last_analyzed,sysdate+10000)),sysdate+10000,'none',min(last_analyzed)), 
      decode(max(nvl(last_analyzed,sysdate-10000)),sysdate-10000,'none',max(last_analyzed)) 
	  into analyzed_oldest, analyzed_newest from dba_tables;
  EXCEPTION WHEN OTHERS THEN
    select null, null into analyzed_oldest, analyzed_newest from dual;
  END;
  BEGIN
    select value into sga_target from v$parameter where name = 'sga_target';
    select value into sga_max_size from v$parameter where name = 'sga_max_size';
   EXCEPTION WHEN OTHERS THEN
    select '0' into sga_target from dual;
    select '0' into sga_max_size from dual;
  END;
  BEGIN
    select value into db_cache_size from v$parameter where name = 'db_cache_size';
    select 0 into db_block_buffers from dual;
  EXCEPTION WHEN OTHERS THEN
    select value into db_block_buffers from v$parameter where name = 'db_block_buffers';
    select 0 into db_cache_size from dual;
  END;
  BEGIN
    select value into log_buffer from v$parameter where name = 'log_buffer';
  EXCEPTION WHEN OTHERS THEN
    select 0 into log_buffer from dual;
  END;
  BEGIN
    select value into db_block_size from v$parameter where name = 'db_block_size';
  EXCEPTION WHEN OTHERS THEN
    select 0 into db_block_size from dual;
  END;
  BEGIN
    select value into db_multiblock from v$parameter where name = 'db_file_multiblock_read_count';
  EXCEPTION WHEN OTHERS THEN
    select 0 into db_multiblock from dual;
  END;
  BEGIN
    select value into shared_pool_size from v$parameter where name = 'shared_pool_size';
  EXCEPTION WHEN OTHERS THEN
    select 0 into shared_pool_size from dual;
  END;
  BEGIN
    select value into large_pool_size from v$parameter where name = 'large_pool_size';
  EXCEPTION WHEN OTHERS THEN
    select 0 into large_pool_size from dual;
  END;
  BEGIN
    select value into java_pool_size from v$parameter where name = 'java_pool_size';
  EXCEPTION WHEN OTHERS THEN
    select 0 into java_pool_size from dual;
  END;
  BEGIN
    select value into workarea_size_policy from v$parameter where name = 'workarea_size_policy';
  EXCEPTION WHEN OTHERS THEN
    select 'n/a' into workarea_size_policy from dual;
  END;
  BEGIN
    select value into pga_aggregate_target from v$parameter where name = 'pga_aggregate_target';
  EXCEPTION WHEN OTHERS THEN
    select 0 into pga_aggregate_target from dual;
  END;
  BEGIN
    select value into filesystemio_options from v$parameter where name = 'filesystemio_options';
  EXCEPTION WHEN OTHERS THEN
    select 'n/a' into filesystemio_options from dual;
  END;
  BEGIN
    select value into disk_asynch_io from v$parameter where name = 'disk_asynch_io';
  EXCEPTION WHEN OTHERS THEN
    select 'n/a' into disk_asynch_io from dual;
  END;

  select substr(value,1,5) into db_version from v$parameter where name = 'compatible';
  IF replace(substr(db_version,1,2),'.','') >= 9 THEN
    BEGIN
      lv_cmd := 'select value from v$pgastat where name = ''cache hit percentage''';
      execute immediate (lv_cmd) into l_pga_hit;
      lv_cmd := 'select min(round(pga_target_for_estimate/1024/1024,0)) from v$pga_target_advice where pga_target_factor > 1 and estd_pga_cache_hit_percentage >= (select least(90,max(estd_pga_cache_hit_percentage)) from v$pga_target_advice)';
      execute immediate (lv_cmd) into l_pga_suggested_mg;
    EXCEPTION WHEN OTHERS THEN
      select 0 into l_pga_hit from dual;
      select 0 into l_pga_suggested_mg from dual;
    END;
  END IF;
  
  BEGIN
    select min(size_for_estimate) into l_db_cache_size_advice
      from  v$db_cache_advice 
      where estd_physical_read_factor < 1
        and name = 'DEFAULT'
      having max(estd_physical_read_factor) <= .99;
    EXCEPTION WHEN OTHERS THEN
      -- Neg 1 is returned if v$db_cache_advice is not populated.
      select nvl(max(-1),0) into l_db_cache_size_advice from v$db_cache_advice where estd_physical_read_factor is null;
  END;

  IF buffer_hit_ratio < 96  
    and trans_rec.total > 50000 
    and instr(&excl_all, database_rec.name)=0 
    and (l_db_cache_size_advice > 0 or l_db_cache_size_advice = -1 ) THEN
    lv_value1 := ltrim(to_char(buffer_hit_ratio,'999.99'));
    INSERT INTO temp_db_checkup_warn VALUES ('Buffer Hit Ratio is '||lv_value1||'%, below 96% and transactions >50K');
    IF l_db_cache_size_advice > 0 THEN
      INSERT INTO temp_db_checkup_warn VALUES ('Buffer Hit Ratio Recommendation: set db_cache_size ='||l_db_cache_size_advice);
    END IF;
  END IF;
  
  BEGIN
    select value into sort_area_size from v$parameter where name = 'sort_area_size';
  EXCEPTION WHEN OTHERS THEN
    select 0 into sort_area_size from dual;
  END;
  BEGIN
    select value into sort_area_retained_size from v$parameter where name = 'sort_area_retained_size';
  EXCEPTION WHEN OTHERS THEN
    select 0 into sort_area_retained_size from dual;
  END;
  BEGIN
    select value into opt_mode from v$parameter where name = 'optimizer_mode';
  EXCEPTION WHEN OTHERS THEN
    select '???' into opt_mode from dual;
  END;

  BEGIN
    select host_name into l_host_name from v$instance;
  END;

  OPEN cur_version;
  FETCH cur_version INTO version;
  OPEN cur_database;
  FETCH cur_database INTO database_rec;
  OPEN cur_sort_stats ;
  FETCH cur_sort_stats INTO disk_sorts, memory_sorts, pct_mem_sorts ;
  OPEN cur_free_memory;
  FETCH cur_free_memory INTO free_memory;
  OPEN cur_buffer_hit_ratio;
  FETCH cur_buffer_hit_ratio INTO buffer_hit_ratio;
  OPEN cur_buffer_wait_ratio;
  FETCH cur_buffer_wait_ratio INTO rec_buffer_wait_ratio;
  OPEN cur_dictionary_hit_ratio;
  FETCH cur_dictionary_hit_ratio INTO dictionary_hit_ratio;
  OPEN cur_library_hit_ratio;
  FETCH cur_library_hit_ratio INTO library_hit_ratio;
  OPEN cur_rbs_online;
  FETCH cur_rbs_online INTO rbs_online_rec;
  OPEN cur_temp_ts;
  FETCH cur_temp_ts INTO temp_ts_rec;
  OPEN cur_trans;
  FETCH cur_trans INTO trans_rec;
  OPEN cur_jobs;
  FETCH cur_jobs INTO jobs_rec;
  OPEN cur_part_ind;
  FETCH cur_part_ind INTO part_ind_rec;
  OPEN cur_modified;
  FETCH cur_modified INTO modified_rec;
  
  INSERT INTO temp_db_checkup VALUES ('Database',database_rec.name,version);
  
  IF sga_target <> '0' THEN
    INSERT INTO temp_db_checkup VALUES ('sga_target',to_char(sga_target/1024/1024,'999,999,999,999'),'Mg (See db_cache_size Advice below)');
    INSERT INTO temp_db_checkup VALUES ('sga_max_size',to_char(sga_max_size/1024/1024,'999,999,999,999'),'Mg');
  ELSE
    IF db_block_buffers > 0 THEN
      INSERT INTO temp_db_checkup VALUES ('db_block_buffers',to_char(db_block_buffers,'999,999,999,999'),'Blocks');
      INSERT INTO temp_db_checkup VALUES ('db_block_size',to_char(db_block_size,'999,999,999,999'),'Bytes (should = or be a multiple of OS block size.)');
      INSERT INTO temp_db_checkup VALUES ('Buffer Memory',to_char(db_block_buffers*db_block_size,'999,999,999,999'),'Bytes (db_block_buffers x db_block_size.)');
    ELSE
      INSERT INTO temp_db_checkup VALUES ('db_block_size',to_char(db_block_size,'999,999,999,999'),'Bytes (should = or be a multiple of OS block size.)');
      INSERT INTO temp_db_checkup VALUES ('Buffer Memory',to_char(db_cache_size,'999,999,999,999'),'Bytes (db_cache_size)');
    END IF;
  
    INSERT INTO temp_db_checkup VALUES ('+ log_buffer',to_char(log_buffer,'999,999,999,999'),'Bytes');
    INSERT INTO temp_db_checkup VALUES ('+ shared_pool_size',to_char(shared_pool_size,'999,999,999,999'),'Bytes');
    INSERT INTO temp_db_checkup VALUES ('+ large_pool_size',to_char(large_pool_size,'999,999,999,999'),'Bytes');
    INSERT INTO temp_db_checkup VALUES ('+ java_pool_size',to_char(java_pool_size,'999,999,999,999'),'Bytes');
  
    IF db_block_buffers > 0 THEN
      INSERT INTO temp_db_checkup VALUES ('SGA',to_char(((db_block_buffers*db_block_size)+log_buffer+shared_pool_size+large_pool_size+java_pool_size)/1024/1024+1,'999,999,999,999'),'Mg (BlkBuffs*BlkSize)+LogBuff+SharedPool+LargePool+JavaPool+1MB');
    ELSE
      INSERT INTO temp_db_checkup VALUES ('SGA',to_char((db_cache_size+log_buffer+shared_pool_size+large_pool_size+java_pool_size)/1024/1024+1,'999,999,999,999'),'Mg (db_cache_size+LogBuff+SharedPool+LargePool+JavaPool+1MB');
    END IF;
  END IF;
  
  INSERT INTO temp_db_checkup VALUES ('db_file_multiblock_read_count',to_char(db_multiblock,'999,999,999,999'),'Blocks');
  INSERT INTO temp_db_checkup VALUES ('Actual Read Size',to_char(db_block_size*db_multiblock,'999,999,999,999'),'Bytes (db_block_size x db_file_multiblock_read_count)');
  
  IF workarea_size_policy <> 'n/a' THEN
    INSERT INTO temp_db_checkup VALUES ('workarea_size_policy',workarea_size_policy,' ');
  END IF;
  
  IF workarea_size_policy <> 'n/a' and workarea_size_policy <> 'AUTO' THEN
     INSERT INTO temp_db_checkup_warn VALUES ('workarea_size_policy is '||workarea_size_policy||' Change to AUTO.');
  END IF;
   
  IF l_pga_hit > 0 THEN
    INSERT INTO temp_db_checkup VALUES ('pga_aggregate_target Hit',to_char(l_pga_hit,'999999999999.99'),'% (Goal is 60%)' );
  END IF;

  IF pga_aggregate_target > 0 THEN
    INSERT INTO temp_db_checkup VALUES ('pga_aggregate_target',to_char(pga_aggregate_target/1024/1024,'999,999,999,999'),'Mg (Def25Mg;No Warn>1G)' );
  END IF;

  IF pga_aggregate_target > 0 and round(pga_aggregate_target/1024/1024,1) <= 1  and l_pga_hit < 60 THEN
    lv_value1 := ltrim(to_char(pga_aggregate_target/1024/1024,'999,999'));
    INSERT INTO temp_db_checkup_warn VALUES ('pga_aggregate_target Hit is '||l_pga_hit||'% Value is: '||lv_value1||'Mg. Recommend: '||l_pga_suggested_mg||'Mg.');
  END IF;  
   
  INSERT INTO temp_db_checkup VALUES ('sort_area_size',to_char(sort_area_size,'999,999,999,999'),'Bytes');
  INSERT INTO temp_db_checkup VALUES ('sort_area_retained_size',to_char(sort_area_retained_size,'999,999,999,999'),'Bytes (Memory each sort retains until the cursor ends.)');
  
  IF sort_area_retained_size > sort_area_size THEN
    lv_value1 := ltrim(to_char(sort_area_retained_size,'999,999,999,999'));
    lv_value2 := ltrim(to_char(sort_area_size,'999,999,999,999'));
    INSERT INTO temp_db_checkup_warn VALUES ('Sort_Area_Retained_Size ('||lv_value1||' Bytes) is larger than Sort_Area_Size ('||lv_value2||' Bytes)');
  END IF;
  
  IF sort_area_size < 65536 THEN
    lv_value1 := ltrim(to_char(sort_area_size,'999,999,999,999'));
    INSERT INTO temp_db_checkup_warn VALUES ('Sort_Area_Size is '||lv_value1||' Bytes. (below 64K) Consider 64KB.');
  END IF;
  
  INSERT INTO temp_db_checkup VALUES ('Disk Sorts',to_char(disk_sorts,'999,999,999,999'),'(Sorts done in TEMP Tablespace.)');
  INSERT INTO temp_db_checkup VALUES ('Memory Sorts',to_char(memory_sorts,'999,999,999,999'),'(Sorts done in sort_area_size Memory.)');
  INSERT INTO temp_db_checkup VALUES ('Ratio',to_char(pct_mem_sorts,'999999999999.99'),'% Sorts done in Memory');
  INSERT INTO temp_db_checkup VALUES ('Free Memory',to_char(free_memory,'999,999,999,999'),'Bytes (No need to increase the shared_pool_size if >2Mb)');
  INSERT INTO temp_db_checkup VALUES ('Buffer Hit Ratio',lpad(to_char(buffer_hit_ratio,'999.99'),16),'% (>95=Good, >99=Excellent, Increase from 95 to 99 can imporve 400%)');
  
  BEGIN
    select min(size_for_estimate) into l_db_cache_size_advice
      from  v$db_cache_advice 
      where estd_physical_read_factor < 1
        and name = 'DEFAULT'
      having max(estd_physical_read_factor) <= .98;
    EXCEPTION WHEN OTHERS THEN
      -- Neg 1 is returned if v$db_cache_advice is not populated.
      select nvl(max(-1),0) into l_db_cache_size_advice from v$db_cache_advice where estd_physical_read_factor is null;
  END;

  IF buffer_hit_ratio < 95  
    and trans_rec.total > 50000 
    and instr(&excl_all, database_rec.name)=0 
    and (l_db_cache_size_advice > 0 or l_db_cache_size_advice = -1 ) THEN
    lv_value1 := ltrim(to_char(buffer_hit_ratio,'999.99'));
    INSERT INTO temp_db_checkup_warn VALUES ('Buffer Hit Ratio is '||lv_value1||'%, below 95% and transactions >50K');
    IF l_db_cache_size_advice > 0 THEN
      INSERT INTO temp_db_checkup_warn VALUES ('Buffer Hit Ratio Recommendation: set db_cache_size ='||l_db_cache_size_advice);
    END IF;
  END IF;

  INSERT INTO temp_db_checkup VALUES ('Buffer Busy Waits',to_char(rec_buffer_wait_ratio.buffer_busy_waits,'999,999,999,999'),' (System Event)');
  INSERT INTO temp_db_checkup VALUES ('Logical Reads',to_char(rec_buffer_wait_ratio.db_block_gets+rec_buffer_wait_ratio.consistent_gets,'999,999,999,999'),' (db block gets + consistent gets)');
  INSERT INTO temp_db_checkup VALUES ('Buffer Wait Ratio',lpad(to_char(rec_buffer_wait_ratio.buffer_busy_waits/(rec_buffer_wait_ratio.db_block_gets+rec_buffer_wait_ratio.consistent_gets)*100,'999.99'),16),'% (<5%=OK)');
  
  IF rec_buffer_wait_ratio.buffer_busy_waits/(rec_buffer_wait_ratio.db_block_gets+rec_buffer_wait_ratio.consistent_gets)*100 > 5 AND trans_rec.total > 50000 THEN
    lv_value1 := ltrim(to_char(rec_buffer_wait_ratio.buffer_busy_waits/(rec_buffer_wait_ratio.db_block_gets+rec_buffer_wait_ratio.consistent_gets)*100,'999.99'));
    INSERT INTO temp_db_checkup_warn VALUES ('Buffer Wait Ratio is '||lv_value1||'%, above 5%');
  END IF;

  INSERT INTO temp_db_checkup VALUES ('Dictionary Hit Ratio',lpad(to_char(dictionary_hit_ratio,'999.99'),16),'% (>95=Good, <95=Increase Shared_Pool_Size unless Free > 2Mb)' );

  IF dictionary_hit_ratio < 96 AND free_memory < 2097152 AND trans_rec.total > 10000 THEN
    lv_value1 := ltrim(to_char(dictionary_hit_ratio,'999.99'));
    lv_value2 := ltrim(to_char(free_memory,'999,999,999,999'));
    INSERT INTO temp_db_checkup_warn VALUES ('Dictionary Hit Ratio is '||lv_value1||'%, (<96%), Free Shared_Pool_Size: '||lv_value2||' Bytes (<2Mg), and Transactions>10K');
  END IF;

  INSERT INTO temp_db_checkup VALUES ('Library Hit Ratio',lpad(to_char(library_hit_ratio,'999.99'),16),'% (>95=Good, <95=Increase Shared_Pool_Size unless Free > 2Mb)' );

  IF library_hit_ratio < 96 AND free_memory < 2097152 THEN
    lv_value1 := ltrim(to_char(library_hit_ratio,'999.99'));
    lv_value2 := ltrim(to_char(free_memory,'999,999,999,999'));
    INSERT INTO temp_db_checkup_warn VALUES ('Library Hit Ratio is '||lv_value1||'%, below 96% and Free Shared_Pool_Size is '||lv_value2||' Bytes, below 2Mg');
  END IF;

  BEGIN
    select free_space, avg_free_size, used_space, avg_used_size, request_failures
      into l_free_space, l_avg_free_size, l_used_space, l_avg_used_size, l_request_failures
    from v$shared_pool_reserved; 
    select value into l_shared_pool_reserved_size from v$parameter where name = 'shared_pool_reserved_size';
    
    -- Default shared_pool_reserved_min is 4400. This is a hidden parameter that can only 
    -- be viewed by SYS
    -- @SU SYS
    -- select x.ksppinm||' = '||y.ksppstvl "# pfile" from sys.x$ksppi x, sys.x$ksppcv y
    -- where x.indx=y.indx
    -- and x.ksppinm like '\_%' escape '\'
    -- and x.ksppinm = '_shared_pool_reserved_min_alloc';    
    
    IF    l_request_failures > 0 and l_last_failure_size > 4400 THEN 
      select 'Increase min_alloc, reserved and shared pool' into l_shared_pool_mod from dual;
    ELSIF l_request_failures >= 0 and l_last_failure_size < 4400 THEN
      select 'Decrease min_alloc and increase shared pool' into l_shared_pool_mod from dual;
    ELSIF l_request_failures > 0 THEN
      select 'Increase shared_pool_size' into l_shared_pool_mod from dual;
    ELSE
      -- Default
      select 'No Request Failures' into l_shared_pool_mod from dual;
    END IF;
    INSERT INTO temp_db_checkup VALUES ('Shared Pool Failures:',lpad(to_char(l_request_failures,'99,999'),16),l_shared_pool_mod);
    IF l_request_failures > 0 THEN
      lv_value1 := ltrim(to_char(l_request_failures,'99,999'));
      lv_value2 := ltrim(to_char(l_shared_pool_reserved_size,'999,999,999,999'));
      INSERT INTO temp_db_checkup_warn VALUES ('Shared Pool Failures:'||lv_value1||'. Reserved Size:'||lv_value1||' '||l_shared_pool_mod);
    END IF;
    EXCEPTION WHEN OTHERS THEN
      null;
  END;

  IF :bv_os = 'AIX' and ( instr(upper(:bv_storage),'NETAPP')>0 
                          or instr(upper(:bv_storage),'N05')>0
                          or instr(upper(:bv_storage),'N06')>0 ) THEN
    INSERT INTO temp_db_checkup VALUES ('OS:',:bv_os_ver,null);
    INSERT INTO temp_db_checkup VALUES ('Storage:',:bv_storage,null);
    INSERT INTO temp_db_checkup VALUES ('filesystemio_options:',filesystemio_options,null);
    INSERT INTO temp_db_checkup VALUES ('disk_asynch_io:',disk_asynch_io,null);
    -- AIX 5.2 on NetApp needs 
    -- filesystemio_options NONE
    -- disk_asynch_io       FALSE
    IF filesystemio_options <> 'n/a' THEN
      IF instr(:bv_os_ver,'5200')>0 THEN
        IF upper(filesystemio_options) <> 'NONE' THEN
          INSERT INTO temp_db_checkup_warn VALUES ('filesystemio_options:'||filesystemio_options||' Should be: NONE on 5.2');
        END IF;
        IF upper(disk_asynch_io) <> 'FALSE'  THEN
          INSERT INTO temp_db_checkup_warn VALUES ('disk_asynch_io:'||disk_asynch_io||' Should be: FALSE on 5.2.');
        END IF;
      END IF;
      -- AIX 5.3 on NetApp needs 
      -- filesystemio_options SETALL
      -- disk_asynch_io       TRUE
      IF instr(:bv_os_ver,'5300')>0 THEN
        IF upper(filesystemio_options) <> 'SETALL' THEN
          INSERT INTO temp_db_checkup_warn VALUES ('filesystemio_options is set to: '||filesystemio_options||' Should be: SETALL on 5.3');
        END IF;
      END IF;
      IF upper(disk_asynch_io) <> 'TRUE'  THEN
        INSERT INTO temp_db_checkup_warn VALUES ('disk_asynch_io is set to: '||disk_asynch_io||' Should be: TRUE on 5.3');
      END IF;
    END IF;
  END IF; 

  lv_value1 := 0;
  BEGIN
    -- Check everything but temporary data files
    FOR non_std_path IN (
        select file_name from
          (select file_name from dba_data_files
           union all
           select name file_name from v$controlfile
           union all 
           select member from v$logfile,v$log where v$logfile.group#=v$log.group#)
        where not exists
          ( select 'x' from v$database 
            where '/'||lower(name)||'/oradata/' = substr(file_name,1,instr(file_name,'/',-1))) ) LOOP
      INSERT INTO temp_db_checkup_warn VALUES ('Non Standard Path: '||non_std_path.file_name);
      lv_value1 := lv_value1 + 1;
    END LOOP;
    -- Check temporary data files which can be stored on a seperate mount.
    FOR non_std_path IN (
        select distinct file_name file_name from dba_temp_files
        where not exists
          ( select 'x' from v$database 
            where '/'||lower(name)||'/oradata/' = substr(file_name,1,instr(file_name,'/',-1))
               or '/'||lower(name)||'_tmp/oradata/' = substr(file_name,1,instr(file_name,'/',-1)))
            ) LOOP
      INSERT INTO temp_db_checkup_warn VALUES ('Non Standard Path: '||non_std_path.file_name);
      lv_value1 := lv_value1 + 1;
    END LOOP;
  END;
  INSERT INTO temp_db_checkup VALUES ('Non Standard Paths:', lpad(lv_value1,16), 'Total');

  lv_value1 := 0;
  BEGIN
    FOR non_std_log IN (
      select member
        from v$logfile ,v$log 
        where v$logfile.group#=v$log.group#
          and substr(substr(member,instr(member,'/',-1)+1),1,4)||substr(member,length(member)-3) <> 'redo.log') LOOP
      INSERT INTO temp_db_checkup_warn VALUES ('Non Standard Log: '||non_std_log.member);
      lv_value1 := lv_value1 + 1;
    END LOOP;
  END;
  INSERT INTO temp_db_checkup VALUES ('Non Standard Logs:', lpad(lv_value1,16), 'Total');
 
  lv_value1 := 0;
  BEGIN
    FOR non_std_ctl IN (
      select name
        from v$controlfile
        where substr(substr(name,instr(name,'/',-1)+1),1,7)||substr(name,length(name)-3) <> 'control.ctl') LOOP
      INSERT INTO temp_db_checkup_warn VALUES ('Non Standard Control File: '||non_std_ctl.name);
      lv_value1 := lv_value1 + 1;
    END LOOP;
  END;
  INSERT INTO temp_db_checkup VALUES ('Non Standard Control Files:', lpad(lv_value1,16), 'Total');

  --Test that local_listener and remote_listener are not set so that the instance 
  --will dynamically register with the listener. 
  BEGIN
    select nvl(value,'NULL') into lv_value1 from v$parameter where name = 'local_listener';
    select nvl(value,'NULL') into lv_value2 from v$parameter where name = 'remote_listener';
    IF lv_value1 <> 'NULL' THEN
      INSERT INTO temp_db_checkup_warn VALUES ('local_listener='||lv_value1||' Should be NULL.');
    END IF;
    IF lv_value2 <> 'NULL' THEN
      INSERT INTO temp_db_checkup_warn VALUES ('remote_listener='||lv_value2||' Should be NULL.');
    END IF;
    INSERT INTO temp_db_checkup VALUES ('local_listener:', lv_value1, null);
    INSERT INTO temp_db_checkup VALUES ('remote_listener:', lv_value2, null);
  END;

  lv_value1 := 0;
  BEGIN
    FOR dup_rec IN (
      select base_name from 
        (select substr(file_name, instr(file_name,'/',-1)+1) base_name from dba_data_files
         union all
         select substr(file_name, instr(file_name,'/',-1)+1) base_name from dba_temp_files)
         group by base_name having count(1) > 1 ) LOOP
      INSERT INTO temp_db_checkup_warn VALUES ('Dup Datafile Name: '||dup_rec.base_name);
      lv_value1 := lv_value1 + 1;
    END LOOP;
  END;
  INSERT INTO temp_db_checkup VALUES ('Dup Datafile Names:', lpad(lv_value1,16), 'Total');

  OPEN cur_system_own;
  WHILE TRUE LOOP
    FETCH cur_system_own INTO lv_owner;
    EXIT WHEN cur_system_own%NOTFOUND;
    INSERT INTO temp_db_checkup_warn VALUES ('User: '||lv_owner||' owns tables in the SYSTEM tablespace.');
  END LOOP;
  lv_comment := to_char(cur_system_own%ROWCOUNT,'999,999,999,999');
  INSERT INTO temp_db_checkup VALUES ('Non SYS Tables in SYSTEM', lv_comment, 'Total');
  CLOSE cur_system_own;

  OPEN cur_system_def;
  WHILE TRUE LOOP
    FETCH cur_system_def INTO lv_username;
    EXIT WHEN cur_system_def%NOTFOUND;
    INSERT INTO temp_db_checkup_warn VALUES ('User: '||lv_username||' has Default_tablespace set to SYSTEM tablespace.');
  END LOOP;
  lv_comment := to_char(cur_system_def%ROWCOUNT,'999,999,999,999');
  INSERT INTO temp_db_checkup VALUES ('Users with SYSTEM TS as default', lv_comment, 'Total');
  CLOSE cur_system_def;

  OPEN cur_system_tmp;
  WHILE TRUE LOOP
    FETCH cur_system_tmp INTO lv_username;
    EXIT WHEN cur_system_tmp%NOTFOUND;
    INSERT INTO temp_db_checkup_warn VALUES ('User: '||lv_username||' has Temporary Tablespace set to SYSTEM tablespace.');
  END LOOP;
  lv_comment := to_char(cur_system_tmp%ROWCOUNT,'999,999,999,999');
  INSERT INTO temp_db_checkup VALUES ('Users with SYSTEM TS as Temp', lv_comment, 'Total');
  CLOSE cur_system_tmp;
  
  OPEN cur_def_passwd;
  WHILE TRUE LOOP
    FETCH cur_def_passwd INTO lv_comment;
    EXIT WHEN cur_def_passwd%NOTFOUND;
    INSERT INTO temp_db_checkup_warn VALUES (lv_comment);
  END LOOP;
  lv_comment := to_char(cur_def_passwd%ROWCOUNT,'999,999,999,999');
  INSERT INTO temp_db_checkup VALUES ('Users with default passwords', lv_comment, 'Total');
  CLOSE cur_def_passwd;

  OPEN cur_dba;
  WHILE TRUE LOOP
    FETCH cur_dba INTO dba_username;
    EXIT WHEN cur_dba%NOTFOUND;
    IF (80 - nvl(length(dba_list),0)) >= ( length(nvl(dba_username,0)) + 1) THEN
      dba_list := dba_list || ' ' || dba_username ;
    END IF;
  END LOOP;
  IF nvl(length(dba_list),0) > 1 THEN
    INSERT INTO temp_db_checkup_warn VALUES ('Unknown DBA users: '||dba_list);
  END IF;
  lv_comment := to_char(cur_dba%ROWCOUNT,'999,999,999,999');
  INSERT INTO temp_db_checkup VALUES ('Unknown DBA users', lv_comment, 'Total');
  CLOSE cur_dba;
  
  lv_value1 := 0;
  BEGIN
     FOR grant_rec IN ( select GRANTEE||' has '||PRIVILEGE  granted  from dba_sys_privs 
                          where PRIVILEGE in ('GRANT ANY ROLE','GRANT ANY PRIVILEGE') 
                          and GRANTEE not in ('DBA','DATAPUMP_IMP_FULL_DATABASE','SPATIAL_WFS_ADMIN_USR','IMP_FULL_DATABASE',
                                              'SPATIAL_CSW_ADMIN_USR','OWB$CLIENT','WKSYS','RECOVERY_CATALOG_OWNER','SYS','OWBSYS')
                          and ( instr(&excl_dba1,GRANTEE)=0 and instr(&excl_dba2,GRANTEE)=0 )) LOOP
      INSERT INTO temp_db_checkup_warn VALUES ('User/Role: '||grant_rec.granted);
      lv_value1 := lv_value1 + 1;
    END LOOP;
  END;
  INSERT INTO temp_db_checkup VALUES ('Users with DBA privs:', lpad(lv_value1,16), 'Total');

  lv_value1 := 0;
  BEGIN
     FOR grant_rec IN ( select GRANTEE||' has '||PRIVILEGE  granted  from dba_sys_privs 
                          where PRIVILEGE in ('CREATE DATABASE LINK') 
                          and GRANTEE not in ('DBA','DATAPUMP_IMP_FULL_DATABASE','SPATIAL_WFS_ADMIN_USR','IMP_FULL_DATABASE',
                                              'SPATIAL_CSW_ADMIN_USR','OWB$CLIENT','WKSYS','RECOVERY_CATALOG_OWNER','SYS','OWBSYS')
                      ) LOOP
      INSERT INTO temp_db_checkup_warn VALUES ('User/Role: '||grant_rec.granted);
      lv_value1 := lv_value1 + 1;
    END LOOP;
  END;
  INSERT INTO temp_db_checkup VALUES ('Users with Link privs:', lpad(lv_value1,16), 'Total');

  BEGIN
    IF instr(&excl_all, database_rec.name)=0 and replace(substr(db_version,1,2),'.','') >= 9 THEN
      -- Skip the Test if this is a Development host.
      IF not ( instr(l_host_name, 'dev') > 0 or instr(l_host_name, 'test') > 0 ) THEN
        select count(1) into lv_value1 from dba_objects 
          where (owner = 'AUTH' and object_name = 'AUTH_CONTROL')
             or (owner = 'SYS' and object_name = 'AUTH_CONNECTION_TRIG');
        IF lv_value1 < 2 THEN
          dbms_output.put_line('AUTH Sechema is not configured');
          INSERT INTO temp_db_checkup_warn VALUES ('AUTH Sechema is not configured');
          INSERT INTO temp_db_checkup VALUES ('Authentication Control', 'Error', 'Status');
        ELSE
          select status into lv_value1 from dba_triggers where owner = 'SYS' and trigger_name ='AUTH_CONNECTION_TRIG'; 
          IF lv_value1 = 'DISABLED' THEN
            INSERT INTO temp_db_checkup_warn VALUES ('SYS.AUTH_CONNECTION_TRIG not enable');
            INSERT INTO temp_db_checkup VALUES ('Authentication Control', 'Error', 'Status');
          ELSE
            lv_cmd := 'select count(1) from auth.auth_control where enabled = ''Y'' and action = ''BLOCK''';
            execute immediate (lv_cmd) into lv_value1;
            -- dbms_output.put_line('lv_value1: '||lv_value1);
            IF lv_value1 < 1 THEN
              INSERT INTO temp_db_checkup_warn VALUES ('No blocking AUTH controls are enabled');
              INSERT INTO temp_db_checkup VALUES ('Authentication Control', 'Error', 'Status');
            ELSE
              INSERT INTO temp_db_checkup VALUES ('Authentication Control', 'OK', 'Status');
            END IF;
          END IF;
        END IF;
      END IF;
    END IF;  
    EXCEPTION WHEN OTHERS THEN
      dbms_output.put_line(substr(SQLERRM, 1, 100));
  END;

  BEGIN
    SELECT 'YES' INTO lv_comment FROM dba_users WHERE username = 'SCOTT';
    EXCEPTION WHEN NO_DATA_FOUND THEN
      SELECT 'NO' INTO lv_comment FROM dual;
  END;
  INSERT INTO temp_db_checkup VALUES ('SCOTT Sample Database Exist:', lv_comment, '');

  OPEN cur_temp_is_perm;
  WHILE TRUE LOOP
    FETCH cur_temp_is_perm INTO lv_tsname;
    EXIT WHEN cur_temp_is_perm%NOTFOUND;
    INSERT INTO temp_db_checkup_warn VALUES ('Tablespace: '||lv_tsname||' contains only Temp segments, but is set to PERMANENT content');
  END LOOP;
  lv_comment := to_char(cur_temp_is_perm%ROWCOUNT,'999,999,999,999');
  INSERT INTO temp_db_checkup VALUES ('Temp Tablespace is PERMANENT', lv_comment, 'Total');
  CLOSE cur_temp_is_perm;

  OPEN cur_temp_perm;
  WHILE TRUE LOOP
    FETCH cur_temp_perm INTO lv_username, lv_tsname;
    EXIT WHEN cur_temp_perm%NOTFOUND;
    INSERT INTO temp_db_checkup_warn VALUES ('User: '||lv_username||' has Temporary Tablespace set to '||lv_tsname||' which contains non-Temp objects.');
  END LOOP;
  lv_comment := to_char(cur_temp_perm%ROWCOUNT,'999,999,999,999');
  INSERT INTO temp_db_checkup VALUES ('Users Temp TS has nontemp objs.', lv_comment, 'Total');
  CLOSE cur_temp_perm;

  BEGIN 
    select instr(&excl_all, database_rec.name) into lv_value1 from dual;
    IF lv_value1 = '0' and instr('PANO',database_rec.name) = '0' and instr('PWP',database_rec.name) = '0'  THEN
      INSERT INTO temp_db_checkup_warn 
        (select decode(max(A.segment_type),'INDEX','alter index ','alter table ')||max(A.owner)||'."'||max(A.segment_name)||
           decode(max(A.segment_type),'INDEX','" rebuild online tablespace ','" move tablespace ') || max(B.tablespace_name) ||
           ' --'||max(A.tablespace_name)||';' 
         from dba_segments A, dba_segments B 
         where A.owner = B.owner 
           and A.segment_type = B.segment_type
           and A.owner not in ('SDE','GDT')
           and A.segment_type not in ('LOBSEGMENT','LOBINDEX')
           and B.tablespace_name like A.owner||'/_%' ESCAPE '/' 
           and A.tablespace_name not like A.owner||'/_%' ESCAPE '/'
           and A.segment_name not like '%PLAN_TABLE%'
         group by A.owner, A.segment_name);
      IF SQL%ROWCOUNT > 0 THEN
        lv_comment := SQL%ROWCOUNT;
      END IF;
      INSERT INTO temp_db_checkup VALUES ('Indexes in Data TS', lv_comment, 'Total');
    END IF;
  END;

  BEGIN
    INSERT INTO temp_db_checkup_warn 
      (select 'Unusable Index: '||owner||'.'||index_name from dba_indexes where status = 'UNUSABLE');
    IF SQL%ROWCOUNT > 0 THEN
      lv_comment := SQL%ROWCOUNT;
    END IF;
    INSERT INTO temp_db_checkup VALUES ('Unusable Indexes: ', lv_comment, 'Total');
  END;

  BEGIN
    select value into rbs_mode from v$parameter where name = 'undo_management';
    EXCEPTION 
    WHEN NO_DATA_FOUND THEN
      select 'Unmanaged' into rbs_mode from dual;
  END;
  INSERT INTO temp_db_checkup VALUES ('RBS Mode', rbs_mode, '');

  IF rbs_mode = 'Unmanaged' THEN

    FOR rbseg_rec IN cur_rbseg LOOP
      IF rbseg_rec.pct_increase = 0 THEN
        IF last_ts <> rbseg_rec.tablespace_name THEN
          IF cur_freebytes%ISOPEN THEN
            CLOSE cur_freebytes;
            CLOSE cur_fixedcost;
          END IF;
          OPEN cur_freebytes (rbseg_rec.tablespace_name) ;
          FETCH cur_freebytes INTO lv_freebytes;
          OPEN cur_fixedcost (rbseg_rec.tablespace_name) ;
          FETCH cur_fixedcost INTO lv_fixedcost;
          last_ts := rbseg_rec.tablespace_name;
        END IF;
        OPEN cur_failafter (rbseg_rec.tablespace_name, rbseg_rec.segment_name, lv_freebytes, lv_fixedcost) ;
        FETCH cur_failafter INTO failafter_rec;
        IF failafter_rec.failafter > failafter_rec.max_extents THEN
          INSERT INTO temp_db_checkup_warn VALUES ('Change Rollback Segment: '||rbseg_rec.segment_name||' MaxExtents to '||ltrim(to_char(failafter_rec.failafter,'999,999,999,999')));
        END IF;
        CLOSE cur_failafter;
      ELSE
        INSERT INTO temp_db_checkup_warn VALUES ('Rollback Segment: '||rbseg_rec.segment_name||' has PCT_INCREASE set to '||ltrim(to_char(rbseg_rec.pct_increase,'999,999,999,999')));
      END IF;
    END LOOP;
    IF cur_freebytes%ISOPEN THEN
      CLOSE cur_freebytes;
      CLOSE cur_fixedcost;
    END IF;

    lv_comment := to_char(rbs_online_rec.online,'999,999,999,999');
    INSERT INTO temp_db_checkup VALUES ('Non-SYS/BIG Online Rollback Sg', lv_comment, 'Total');
    IF rbs_online_rec.online < 2 THEN
      INSERT INTO temp_db_checkup_warn VALUES ('Less than 2 Non-SYSTEM Rollback Segments Online.');
    END IF;

    lv_comment := to_char(rbs_online_rec.active,'999,999,999,999');
    INSERT INTO temp_db_checkup VALUES ('Non-SYS/BIG Active Rollback Sg', lv_comment, 'Total');
    IF rbs_online_rec.active = rbs_online_rec.online THEN
      INSERT INTO temp_db_checkup_warn VALUES ('All '||ltrim(lv_comment)||' Non-SYSTEM Online Rollback Segments are Active.');
    END IF;

    lv_comment := to_char(rbs_online_rec.totalwaits,'999,999,999,999');
    INSERT INTO temp_db_checkup VALUES ('Non-SYS/BIG Rollback Sg Wait', lv_comment, 'Total');
    IF rbs_online_rec.totalwaits > 10 THEN
      INSERT INTO temp_db_checkup_warn VALUES ('Total Waits for Non-SYSTEM Online Rollback Segments is '||ltrim(lv_comment)||' (over 10) Consider adding more RBS (up to 50)');
    END IF;

    lv_value1 := 0;
    FOR rbs_share_rec IN cur_rbs_share LOOP
      IF rbs_share_rec.err > 0 THEN
        INSERT INTO temp_db_checkup_warn VALUES ('Tablespace: '||rbs_share_rec.tablespace||' contains RBS segments and Non-RBS Segment Types.');
        lv_value1 := lv_value1 + 1;
      END IF;
    END LOOP;
    INSERT INTO temp_db_checkup VALUES ('RBS TS with Non-RBS Segments', lpad(lv_value1,16), 'Total');

  ELSE
    -- rbs_mode is MANUAL or AUTO
    select substr(value,1,5) into db_version from v$parameter where name = 'compatible';
    IF replace(substr(db_version,1,2),'.','') >= 9 THEN
      BEGIN
        lv_cmd := 'select sum(NoSpaceErrCnt) from v$undostat';
        execute immediate (lv_cmd) into NoSpaceErrCnt;
        lv_cmd := 'select to_char(min(begin_time),''Mon DD HH24:MI'') from v$undostat';
        execute immediate (lv_cmd) into BeginNoSpace;
      EXCEPTION WHEN OTHERS THEN
        select 'n/a' into NoSpaceErrCnt from dual;
        select 'n/a' into BeginNoSpace from dual;
      END;
      IF NoSpaceErrCnt > 5 THEN
        INSERT INTO temp_db_checkup_warn VALUES ('UNDO No Space Error Count: '||ltrim(NoSpaceErrCnt)||' Above 5 since '||BeginNoSpace);
      END IF;        
      INSERT INTO temp_db_checkup VALUES ('UNDO No Space Err Cnt', lpad(NoSpaceErrCnt,16), 'Total since '||BeginNoSpace);
    END IF;      
  END IF;  --Unmanaged RBS

  lv_value1 := 0;
  BEGIN
    FOR ext_space_rec IN 
         (select a.tablespace_name, a.allocated_megs, a.max_extent_megs, a.extend_avail_megs, b.free_megs
          from
          (select tablespace_name, 
             sum(bytes/1024/1024) allocated_megs,
             sum(maxbytes/1024/1024) max_extent_megs,
             sum((decode(maxbytes,0,bytes,maxbytes)-bytes)/1024/1024) extend_avail_megs
           from dba_data_files
           where   tablespace_name not like 'RBS%'
             and tablespace_name not like '%UNDOT%'
             and tablespace_name not in ('TEMP','TOOLS','USERS','DRSYS')
           group by tablespace_name) a,  
           (select tablespace_name,
              round(sum(bytes)/1024/1024,0) free_megs 
            from dba_free_space 
            group by tablespace_name) b
           where a.tablespace_name = b.tablespace_name (+)
             and ( a.extend_avail_megs + nvl(b.free_megs,0) < (decode(database_rec.name,'PANO',1000,'MAX',2000,'TMAXSUP',2000,'DMAXSUP',2000,'OTFS',2000,'FLOW',2000,'OAPRD1',600,200))
             and nvl(b.free_megs,0) < .2 * a.allocated_megs) 
             and a.tablespace_name not like 'RBS%'
            and a.tablespace_name not like '%UNDOT%'
            and a.tablespace_name not in ('TEMP','TOOLS','USERS','DRSYS') ) LOOP
      INSERT INTO temp_db_checkup_warn VALUES ('TS: '||ext_space_rec.tablespace_name||' Alloc:'||ext_space_rec.allocated_megs||'Mg. Auto Extend + Free Avail:'||(ext_space_rec.extend_avail_megs+ext_space_rec.free_megs)||'Mg  (<'||decode(database_rec.name,'PANO',1000,'MAX',2000,'TMAXSUP',2000,'DMAXSUP',2000,'OTFS',2000,'FLOW','2000','OAPRD1','600','200')||')');
      lv_value1 := lv_value1 + 1;
    END LOOP;
    INSERT INTO temp_db_checkup VALUES ('TS w/<'||decode(database_rec.name,'PANO',1000,'MAX',2000,'TMAXSUP',2000,'DMAXSUP',2000,'OTFS',2000,'FLOW','2000','OAPRD1','600','200')||'MB Auto Extend + Free Space Avail', lv_value1, 'Total');
  END;

  OPEN cur_cached_cursors;
  FETCH cur_cached_cursors INTO cached_cursors_rec;
  INSERT INTO temp_db_checkup VALUES ('session_cached_cursors Usage',lpad(to_char(cached_cursors_rec.usage,'999'),16), '% Used: '||cached_cursors_rec.used||' Available: '||cached_cursors_rec.value );
  IF cached_cursors_rec.usage > 95 THEN
    INSERT INTO temp_db_checkup_warn VALUES ('session_cached_cursors usage:'||to_char(cached_cursors_rec.usage,'999')||'% (>90% set session_cached_cursors = '||round(cached_cursors_rec.used*1.2,0)||' )');
  END IF;  
  CLOSE cur_cached_cursors;
  
  OPEN cur_open_cursors;
  FETCH cur_open_cursors INTO open_cursors_rec;
  INSERT INTO temp_db_checkup VALUES ('open_cursors Usage',lpad(to_char(open_cursors_rec.usage,'999'),16), '% Used: '||open_cursors_rec.used||' Available: '||open_cursors_rec.value );
  IF open_cursors_rec.usage > 85 THEN
    INSERT INTO temp_db_checkup_warn VALUES ('open_cursors usage:'||to_char(open_cursors_rec.usage,'999')||'% (Increase open_cursors)');
  END IF;
  CLOSE cur_open_cursors;
  
  OPEN cur_invalid_obj;
  WHILE TRUE LOOP
    FETCH cur_invalid_obj INTO invalid_obj_rec;
    EXIT WHEN cur_invalid_obj%NOTFOUND;
    lv_value1 := ltrim(to_char(invalid_obj_rec.cnt,'999,999,999'));
    lv_comment := ' Invalid Objects for owner '||invalid_obj_rec.owner||'. See details below.' ;
    INSERT INTO temp_db_checkup_warn VALUES (lv_value1||lv_comment);
  END LOOP;
  lv_comment := to_char(cur_invalid_obj%ROWCOUNT,'999,999,999,999');
  INSERT INTO temp_db_checkup VALUES ('Owners with Invalid Objects', lv_comment, 'Total');
  CLOSE cur_invalid_obj;

  OPEN cur_invalid_syn;
  WHILE TRUE LOOP
    FETCH cur_invalid_syn INTO invalid_syn_rec;
    EXIT WHEN cur_invalid_syn%NOTFOUND;
    lv_value1 := ltrim(to_char(invalid_syn_rec.cnt,'999,999,999'));
    lv_comment := ' Phantom Synonymn for owner '||invalid_syn_rec.owner||' Example: '||invalid_syn_rec.synonym_name ;
    INSERT INTO temp_db_checkup_warn VALUES (lv_value1||lv_comment);
  END LOOP;
  lv_comment := to_char(cur_invalid_syn%ROWCOUNT,'999,999,999,999');
  INSERT INTO temp_db_checkup VALUES ('Owners with Phantom Synonyms', lv_comment, 'Total');
  CLOSE cur_invalid_syn;

  INSERT INTO temp_db_checkup VALUES ('Broken or Failed Jobs',lpad(to_char(jobs_rec.failed,'999'),16), '');
  IF jobs_rec.failed > 0 THEN
      INSERT INTO temp_db_checkup_warn VALUES ('Broken or Failed Jobs, Count:'||jobs_rec.failed||' (See details below)');
  END IF;  

  INSERT INTO temp_db_checkup VALUES ('Unusable Partition Indexes: ',lpad(to_char(part_ind_rec.cnt,'999'),16), '');
  IF part_ind_rec.cnt > 0 THEN
      INSERT INTO temp_db_checkup_warn VALUES ('Unusable Partition Indexes: '||part_ind_rec.cnt||' (See details below)');
  END IF;  

  INSERT INTO temp_db_checkup VALUES ('Objs Modified in past 24 Wrk Hrs: ',lpad(to_char(modified_rec.cnt,'999'),16), '');
  --IF modified_rec.cnt > 0 THEN
  --    INSERT INTO temp_db_checkup_warn VALUES ('Objs Modified in past 24 Wrk Hrs: '||modified_rec.cnt||' (See details below)');
  --END IF;  

  lv_value1 := 0;
  FOR backup_ts_rec IN cur_backup_ts LOOP
    INSERT INTO temp_db_checkup_warn VALUES ('Tablespace: '||backup_ts_rec.tablespace_name||' is in backup mode.');
    lv_value1 := lv_value1 + 1;
  END LOOP;
  INSERT INTO temp_db_checkup VALUES ('TS in backup mode', lpad(lv_value1,16), 'Total');
  INSERT INTO temp_db_checkup VALUES ('optimizer_mode', opt_mode, '');
  -- Not avail in version 7
  INSERT INTO temp_db_checkup VALUES ('Oldest Analyzed Table Date', to_char(analyzed_oldest,'Mon DD'), '');
  INSERT INTO temp_db_checkup VALUES ('Newest Analyzed Table Date', to_char(analyzed_newest,'Mon DD'), '');
  IF analyzed_newest < sysdate-15 AND instr(&excl_analyze, database_rec.name)=0 and l_host_name not like ('%test%') THEN
    lv_value1 := to_char(analyzed_newest,'Day Mon DD');
    INSERT INTO temp_db_checkup_warn VALUES ('Tables have not been analyzed since '||lv_value1||' (>15 days)');
  END IF;
  INSERT INTO temp_db_checkup VALUES ('TEMP Tablespace Free',lpad(to_char(temp_ts_rec.pct,'999.99'),16),'% of '||to_char(temp_ts_rec.avail)||' Mb available.' );
  IF temp_ts_rec.pct < 10 THEN
      lv_value1 := ltrim(to_char(temp_ts_rec.pct,'999.99'))||'% of '||to_char(temp_ts_rec.avail)||' Mb available.';
      INSERT INTO temp_db_checkup_warn VALUES ('Free TEMP Tablespace is '||lv_value1||' (Less than 10%)');
  END IF;
  
  INSERT INTO temp_db_checkup VALUES ('Transactions',to_char(trans_rec.total,'999,999,999,999'),trans_rec.comments);
      
  CLOSE cur_database;
  CLOSE cur_version;
  CLOSE cur_sort_stats ;
  CLOSE cur_buffer_hit_ratio;
  CLOSE cur_buffer_wait_ratio;
  CLOSE cur_dictionary_hit_ratio;
  CLOSE cur_library_hit_ratio;
  CLOSE cur_rbs_online;
  CLOSE cur_temp_ts;
  CLOSE cur_trans;  
  CLOSE cur_jobs;
  CLOSE cur_part_ind;
  CLOSE cur_modified;
  
END;
/

-- Archive Mode stuff
DECLARE
CURSOR cur_database is
  select NAME, LOG_MODE from v$database;
database_rec cur_database%ROWTYPE;  
CURSOR cur_archive is
  select a.value archive_start, b.value ckpnt_interval, c.value ckpnt_timeout
  from v$parameter a, v$parameter b, v$parameter c 
  where a.name = 'log_archive_start'
  and b.name = 'log_checkpoint_interval'
  and c.name = 'log_checkpoint_timeout';
archive_rec cur_archive%ROWTYPE;
CURSOR cur_version is
  select substr(value,1,5) version from v$parameter where name = 'compatible';
lc_version varchar2(5);
-- Declare first_time to prevent fail of cur_loghist in Version 7 databases and time for Version 8.
first_time date;
time varchar2(20);
CURSOR cur_loghist is
  select to_char(first_time,'HH12:MI AM') time,
  (to_number(to_char(first_time,'HH24'))*60)+(to_number(to_char(first_time,'MI'))) minutes
  from v$log_history where to_char(sysdate,'YYYYMMDD') = to_char(first_time,'YYYYMMDD')
  order by time;
loghist_rec cur_loghist%ROWTYPE;
CURSOR cur_loghist_v7 is
  select substr(time,10,5) time,
  (to_number(substr(time,10,2))*60)+(to_number(substr(time,13,2))) minutes
  from v$log_history
  where to_char(sysdate,'MM/DD/YY') = substr(time,1,8)
  order by time;
loghist_rec_v7 cur_loghist_v7%ROWTYPE;
CURSOR cur_redologs is
  select substr(a.member,1,40) member, 
    to_char(b.bytes/1024/1024,'999,999')||to_char(b.members,'999')||lpad(b.archived,4)||' '||b.status MMArcStat
    from v$logfile a, v$log b where a.group# = b.group#;
redologs_rec cur_redologs%ROWTYPE;

CURSOR cur_dis_const is
  select c.owner as owner, to_char(count(1)) as const_count 
    from dba_constraints c, v$instance i
    where i.host_name not in ('rsprod8','rstest7') 
      and c.status = 'DISABLED' 
      and c.owner not in ('OLAPSYS','SYSTEM','SYS') group by owner;
dis_const_rec cur_dis_const%ROWTYPE;
CURSOR cur_dis_trig is
  /* Ignore APWX */
  select owner as owner, to_char(count(1)) as trig_count 
    from dba_triggers where owner not in ('SYS','SYSTEM','EXFSYS','WMSYS','APWX') and status = 'DISABLED' and instr(&excl_trig,trigger_name)=0 group by owner;
dis_trig_rec cur_dis_trig%ROWTYPE;

ln_first number := 0;
lc_first_time varchar(8) := '';
ln_last number := 0;
lc_last_time varchar(8) := '';
ln_lapse number := 0;
ln_cur number := (to_number(to_char(sysdate,'HH24'))*60)+(to_number(to_char(sysdate,'MI')));
ln_sum number := 0;
ln_count number := 0;
ln_avg_today number := 0;
lc_lapse varchar2(1000) := '';
lv_comment varchar2(120) := '';
BEGIN
  OPEN cur_version;
  FETCH cur_version INTO lc_version;

  OPEN cur_database;
  FETCH cur_database INTO database_rec;
  OPEN cur_archive;
  FETCH cur_archive INTO archive_rec;
  IF database_rec.log_mode = 'NOARCHIVELOG' THEN
    lv_comment := 'Not in archivelog mode.';
    INSERT INTO temp_db_checkup VALUES ('Log Mode', database_rec.log_mode, lv_comment);
  ELSE
    lv_comment := 'Check for "Checkpoint not complete" in Alter log.';
    IF archive_rec.archive_start = 'FALSE' and substr(lc_version,1,2) < 10 THEN
      lv_comment := 'ARCHIVE NOT STARTED!';
      INSERT INTO temp_db_checkup_warn VALUES ('Database is in Archive Mode but Archive Start is FALSE!');
    END IF;
    INSERT INTO temp_db_checkup VALUES ('log_archive_start', 'TRUE', lv_comment);
    INSERT INTO temp_db_checkup VALUES ('log_checkpoint_interval', lpad(archive_rec.ckpnt_interval,16), 'operating system blocks, normally 512K (default is 10,000 [5M @ 512K])');
    INSERT INTO temp_db_checkup VALUES ('log_checkpoint_timeout', lpad(archive_rec.ckpnt_timeout,16), 'seconds (default is 1800)');
   
    -- Cursor: cur_version
    IF substr(lc_version,1,1) = '7' THEN 
      -- Version 7 Database
      FOR loghist_rec_v7 IN cur_loghist_v7 LOOP
        IF ln_first = 0 THEN
          ln_first := loghist_rec_v7.minutes;
          lc_first_time := loghist_rec_v7.time; 
          ln_last := loghist_rec_v7.minutes;
        ELSE
          ln_lapse := loghist_rec_v7.minutes - ln_last;
          ln_sum := ln_sum + ln_lapse;
          ln_count := ln_count + 1;
          lc_lapse := lc_lapse||','||to_char(ln_lapse);
          ln_last := loghist_rec_v7.minutes;
          lc_last_time := loghist_rec_v7.time; 
        END IF;
      END LOOP;
    ELSE
      -- Version 8 and up
      FOR loghist_rec IN cur_loghist LOOP
        IF ln_first = 0 THEN
          ln_first := loghist_rec.minutes;
          lc_first_time := loghist_rec.time; 
          ln_last := loghist_rec.minutes;
        ELSE
          ln_lapse := loghist_rec.minutes - ln_last;
          ln_sum := ln_sum + ln_lapse;
          ln_count := ln_count + 1;
          lc_lapse := lc_lapse||','||to_char(ln_lapse);
          ln_last := loghist_rec.minutes;
          lc_last_time := loghist_rec.time; 
        END IF;
      END LOOP;
    END IF;
    IF ln_last <> 0 THEN
      ln_lapse := ln_cur - ln_last;
    END IF;
    INSERT INTO temp_db_checkup VALUES ('Archives Today', lpad(ln_count,16), 'Total');
    INSERT INTO temp_db_checkup VALUES ('Archives Today; First', lpad(lc_first_time,16), '');
    INSERT INTO temp_db_checkup VALUES ('Archives Today; Last', lpad(lc_last_time,16), '');
    INSERT INTO temp_db_checkup VALUES ('Archive Intervals in Minutes', substr(lc_lapse,2,17), substr(lc_lapse,18,88));
    INSERT INTO temp_db_checkup VALUES ('Avg Minutes between Archives', lpad(to_char(ln_sum/greatest(1,ln_count),'999,999'),16), 'Min.');
    INSERT INTO temp_db_checkup VALUES ('Minutes since last archive', lpad(to_char(ln_lapse),16), 'Min.');
    IF ln_count > 50 and (ln_sum/ln_count)<5 THEN
      lv_comment := to_char(ln_count)||' Archives (Over 50) have be written today and the Avg Time between Archives is ';
      INSERT INTO temp_db_checkup_warn VALUES (lv_comment||ltrim(to_char(ln_sum/ln_count,'999,999.9'))||' minutes. (Under 5)');
    END IF;
  END IF;

  INSERT INTO temp_db_checkup VALUES ('','','');
  INSERT INTO temp_db_checkup VALUES ('Redo Member', '', '      Mb Mem Arc Status');
  FOR redologs_rec IN cur_redologs LOOP
    lv_comment := length(ltrim(redologs_rec.member))-20;
    INSERT INTO temp_db_checkup VALUES (substr(redologs_rec.member,1,6)||'...'||substr(redologs_rec.member,lv_comment,200),'',redologs_rec.MMArcStat);
  END LOOP;

  FOR dis_const_rec IN cur_dis_const LOOP
    IF cur_dis_const%ROWCOUNT = 1 THEN
      INSERT INTO temp_db_checkup_warn VALUES ('1 or more constraints are disabled. See details below.');
    END IF;
    -- INSERT INTO temp_db_checkup VALUES (substr(dis_const_rec.owner,1,60),'',dis_const_rec.const_count);
  END LOOP;

  FOR dis_trig_rec IN cur_dis_trig LOOP
    IF cur_dis_trig%ROWCOUNT = 1 THEN
      INSERT INTO temp_db_checkup_warn VALUES ('1 or more triggers are disabled. See details below.');
    END IF;
  END LOOP;

  CLOSE cur_database;
  CLOSE cur_archive;
  
END;
/

SET PAGESIZE 600;
SET LINESIZE 120;

column warning format a80 heading 'WARNING:'
SELECT warning from temp_db_checkup_warn;

column setting       format a30        heading 'Setting'
column value         format a16        heading 'Value'
column comments      format a70        heading 'Comments'
SELECT lpad(setting,30) as  setting, value, comments
  from temp_db_checkup;

--clear columns
--SELECT lpad(setting,30) as "Setting", substr(value,1,16) as "Value", comments as "Comments"
--  from temp_db_checkup;

set heading off
select decode(count(1),0,'',CHR(10)||'DISABLED CONSTRAINTS:') from dba_constraints c, v$instance i
  where i.host_name not in ('rsprod8','rstest7')
  and c.status='DISABLED'
  and c.owner not in ('OLAPSYS','SYSTEM');
set heading on
select substr(c.owner,1,10) "Owner", substr(c.table_name,1,25) "Table Name", c.constraint_name "Constraint Name",
  decode(c.constraint_type,'C','Check','O','Other','P','Primary Key','R','Foreign Key','U','Unique','Unknown') "Type",
  substr(decode(c.r_owner,'','',c.r_owner||'.'||c.r_constraint_name),1,30)  "Referencing"
  from dba_constraints c, v$instance i
  where i.host_name not in ('rsprod8','rstest7')
    and c.status='DISABLED' and c.owner not in ('OLAPSYS','SYSTEM')
  order by 1,2;

set heading off
select decode(count(1),0,'',CHR(10)||'DISABLED TRIGGERS:') from dba_triggers where owner not in ('SYS','SYSTEM','EXFSYS','APWX') and status='DISABLED' and instr(&excl_trig,trigger_name)=0;
set heading on
select substr(owner,1,10) "Owner", substr(table_name,1,25) "Table Name", trigger_name "Trigger Name"
  from dba_triggers
  where owner not in ('SYS','SYSTEM','EXFSYS','APWX') and status='DISABLED' and instr(&excl_trig,trigger_name)=0
  order by 1,2;

set heading off
select decode(count(1),0,'',CHR(10)||'INVALID OBJECTS:') from dba_objects where status='INVALID' and object_type not in ('UNDEFINED');
set heading on
select substr(owner,1,15) "Owner", substr(object_type,1,12) "Type",
  substr(object_name,1,35) "Invalid Objects" from dba_objects 
  where status='INVALID' and object_type not in ('UNDEFINED') ORDER by 1,2,3;

set heading off
select decode(count(1),0,'',CHR(10)||'BROKEN or FAILED JOBS:') from dba_jobs where broken = 'Y' or failures > 0;
set heading on
select /*+ RULE */ substr(j.job,1,3) job, decode(r.job,null,'       ','RUNNING') RUNNING,
 substr(j.schema_user,1,10) schema, j.broken, substr(j.failures,1,4) "FAIL", 
 to_char(j.next_date,'MON DD HH24:MI') Next, substr(j.what,1,65) what 
 from dba_jobs j, dba_jobs_running r
 where j.job = r.job (+)
   and (j.broken = 'Y' or j.failures > 0)
 order by j.next_date;

set heading off
select decode(count(1),0,'',CHR(10)||'Objects Modified in the past 24 hours:') 
  from dba_objects o, v$instance i
  where i.host_name not in ('rsprod8','rstest7') 
    and o.last_ddl_time >= sysdate-1 and o.owner not in ('SYSTEM','SYS','PUBLIC','DJACKSON');
set heading on
column owner format a15 heading "Owner"
column object_type format a18 heading "Type"
column object_name format a40 heading "Name"
column last_ddl_time format a20 heading "Modified"
select o.owner, o.object_type, o.object_name, to_char(o.LAST_DDL_TIME, 'DAY HH:MM AM') last_ddl_time
  from dba_objects o, v$instance i
  where i.host_name not in ('rsprod8','rstest7') 
    and last_ddl_time >= sysdate-1
    and owner not in ('SYS','SYSTEM','DJACKSON')
  order by owner, object_type, object_name; 

-- Schema Owners
set heading on
select distinct o.owner as "Owners:" from dba_objects o, v$instance i
  where i.host_name not in ('rsprod8','rstest7') 
    and o.owner not in ('SYS','SYSTEM','OUTLN','WMSYS','OEM_MONITOR','LOGSTDBY_ADMINISTRATOR','CTXSYS','EXFSYS','MDDATA','MDSYS','OLAPSYS','OLAP_USER','ORDPLUGINS','ORDSYS','SI_INFORMTN_SCHEMA','WKSYS','XDB','DBSNMP','DJACKSON','BRC','DMSYS','PUBLIC','SYSMAN','WK_TEST')
  order by owner;

-- RESOURCE role users  
set heading on
select grantee as "Resouce Users:" from dba_role_privs where granted_role = 'RESOURCE'
  and grantee not in ('SYS','SYSTEM','OUTLN','WMSYS','OEM_MONITOR','LOGSTDBY_ADMINISTRATOR','CTXSYS','EXFSYS','MDDATA','MDSYS','OLAPSYS','OLAP_USER','ORDPLUGINS','ORDSYS','SI_INFORMTN_SCHEMA','WKSYS','XDB')
  order by grantee;


prompt
prompt SGA Details
compute sum label 'SGA:' of kb on report
break on report
column name heading 'Parameter' format a30
column kb heading 'KB' format 999,999,999,999
column min_KB heading 'Minumum KB'
/* If sga_target is being used then we get the hidden parameters */
select a.name, a.KB, to_char(b.min_KB,'999,999,999') min_KB from
  (select name, value/1024 KB, '' min_KB from
    (select x.ksppinm name, y.ksppstvl value from sys.x$ksppi x, sys.x$ksppcv y
     where x.indx=y.indx
     and x.ksppinm like '\_%' escape '\')
  where value <> '0' and name in
    ('__shared_pool_size','__java_pool_size','__streams_pool_size','__log_buffer',
     '__db_cache_size','__db_2k_cache_size','__db_4k_cache_size','__db_8k_cache_size',
     '__db_16k_cache_size','__db_32k_cache_size','__db_keep_cache_size',
     '__db_recycle_cache_size','__large_pool_size')) a,
  (select '__'||name min_name, value/1024 min_KB
    from v$parameter
    where value <> '0'
    and name in
    ('buffer_pool_keep','buffer_pool_recycle','global_context_pool_size',
     'java_pool_size','large_pool_size','shared_pool_size','db_16k_cache_size',
     'db_2k_cache_size','db_32k_cache_size','db_4k_cache_size','db_8k_cache_size',
     'db_cache_size','db_keep_cache_size','db_recycle_cache_size','log_buffer')
    and name in
    (select substr(name,3) from
      (select x.ksppinm name, y.ksppstvl value from sys.x$ksppi x, sys.x$ksppcv y
       where x.indx=y.indx
       and x.ksppinm like '\_%' escape '\')
     where exists (select 'x' from v$parameter where name = 'sga_target' and value <> '0')
     and value <> '0' and name in
     ('__shared_pool_size','__java_pool_size','__streams_pool_size','__log_buffer',
     '__db_cache_size','__db_2k_cache_size','__db_4k_cache_size','__db_8k_cache_size',
     '__db_16k_cache_size','__db_32k_cache_size','__db_keep_cache_size',
     '__db_recycle_cache_size','__large_pool_size'))) b
where a.name  = b.min_name (+)
union
/* Get manually configured parameters less any auto parameter if sga_target is being used.*/
/* for 8i */
select 'buffers_x_block_size' name, b.block_buffers * s.block_size /1024 KB, '' min_KB  from
  (select value block_buffers from v$parameter where name = 'db_block_buffers') b,
  (select value block_size from v$parameter where name = 'db_block_size') s
  where exists (select 'x' from v$parameter where name = 'db_block_buffers' and value <>'0')
union
/* for 8i and up */
select name,
  /* 8i reports K in the value of some parameters */
  round(decode(instr(value,'K'),0,value,replace(value,'K','')*1024))/1024 KB, '' min_KB
  from v$parameter
  where value <> '0'
  and name in
  ('buffer_pool_keep','buffer_pool_recycle','global_context_pool_size',
   'java_pool_size','large_pool_size','shared_pool_size','db_16k_cache_size',
   'db_2k_cache_size','db_32k_cache_size','db_4k_cache_size','db_8k_cache_size',
   'db_cache_size','db_keep_cache_size','db_recycle_cache_size','log_buffer')
  and name not in
  (select substr(name,3) name from
    (select x.ksppinm name, y.ksppstvl value from sys.x$ksppi x, sys.x$ksppcv y
     where x.indx=y.indx
     and x.ksppinm like '\_%' escape '\')
   where exists (select 'x' from v$parameter where name = 'sga_target' and value <> '0')
   and value <> '0' and name in
   ('__shared_pool_size','__java_pool_size','__streams_pool_size','__log_buffer',
   '__db_cache_size','__db_2k_cache_size','__db_4k_cache_size','__db_8k_cache_size',
   '__db_16k_cache_size','__db_32k_cache_size','__db_keep_cache_size',
   '__db_recycle_cache_size','__large_pool_size')
  );
clear break
clear columns

-- Create dummy table for <9i.
DECLARE
  v_check number;
  cmd varchar2(500);
BEGIN
  cmd:='select count(1) from v$db_cache_advice';
  execute immediate (cmd) into v_check;
  EXCEPTION WHEN OTHERS THEN
    BEGIN
      cmd:='create table v$db_cache_advice (size_for_estimate number,buffers_for_estimate number, estd_physical_read_factor number, estd_physical_reads number)';
      execute immediate (cmd);
    END;
END;
/
prompt
prompt DB Cache Advice
clear columns
set head on
column size_for_estimate format 999,999,999,999 heading 'Cache Size (m)'
column buffers_for_estimate format 999,999,999 heading 'Buffers'
column estd_physical_read_factor format 999.90 heading 'Estd Phys|Read Factor'
column estd_physical_reads format 999,999,999 heading 'Estd Phys| Reads'
select size_for_estimate, buffers_for_estimate, estd_physical_read_factor, estd_physical_reads
from v$db_cache_advice where name = 'DEFAULT'
and block_size = (select value from v$parameter where name = 'db_block_size') and advice_status = 'ON';
prompt
-- Drop the dummy table;
BEGIN
  execute immediate ('drop table v$db_cache_advice');
  EXCEPTION WHEN OTHERS THEN
    null;
END;
/


-- Create dummy table for <9i.
DECLARE
  v_check number;
  cmd varchar2(500);
BEGIN
  cmd:='select count(1) from v$pga_target_advice';
  execute immediate (cmd) into v_check;
  EXCEPTION WHEN OTHERS THEN
    BEGIN
      cmd:='create table v$pga_target_advice (pga_target_for_estimate number,pga_aggregate_target number, pga_target_factor number, estd_pga_cache_hit_percentage number)';
      execute immediate (cmd);
    END;
END;
/
prompt
prompt PGA_AGGREGATE_TARGET Advice
clear columns
column name format a35
set head off
select name,value from v$sysstat where name like 'workarea executions%';
set head on
column pga_aggregate_target heading "pga_aggregate_target|(MB)"
column pga_target_factor format 99.99 heading "Factor"
column estd_pga_cache_hit_percentage format 999 heading "Estd Cache|Hit %"
select round(pga_target_for_estimate/1024/1024,0) pga_aggregate_target,
  pga_target_factor, estd_pga_cache_hit_percentage 
  from v$pga_target_advice;
prompt
-- Drop the dummy table;
BEGIN
  execute immediate ('drop table v$pga_target_advice');
  EXCEPTION WHEN OTHERS THEN
    null;
END;
/

prompt
prompt Tables with greater then 10 percent chained rows when no Long fields exist.
column table_name format a45 heading "Table"
column num_rows heading "Num Rows"
column chain_cnt heading "Chain Cnt"
column pct_free heading "PCTFREE"
column Pct heading "Pct"
select owner||'.'||table_name table_name, num_rows, chain_cnt, pct_free, round(decode(num_rows,0,0,(chain_cnt/num_rows)*100),0) Pct
from dba_tables 
where decode(num_rows,0,0,(chain_cnt/num_rows)*100)  >= 10
and owner||'.'||table_name not in (select distinct owner||'.'||table_name from dba_tab_columns where data_type like 'LONG%')
order by 1,4 desc;
prompt

prompt
prompt Top 10 Fragmented Tablespaces
select * from 
(
select fs.tablespace_name, extents, datafiles from 
(select tablespace_name, count(1) extents from dba_free_space group by tablespace_name) fs, 
(select tablespace_name, count(1) datafiles from dba_data_files group by tablespace_name) df
where fs.tablespace_name = df.tablespace_name
and extents-datafiles > 0
order by extents-datafiles desc
) where rownum <11;
prompt

-- Create dummy table for <9i.
DECLARE
  v_check number;
  cmd varchar2(500);
BEGIN
  cmd:='select count(1) from v$sql_plan where rownum <2';
  execute immediate (cmd) into v_check;
  EXCEPTION WHEN OTHERS THEN
    BEGIN
    cmd:='create table v$sql_plan (address char(1), object_owner char(1), object_name char(1), operation char(1), options char(1))'; 
    execute immediate (cmd);
    END;
END;
/
column exec         format 99,999,999 
column num_rows     format 99,999,999,999
column total_rows_K format 999,999,999,999
column name         format a40;
column ch           format a1;
prompt Top 20 Full Table Scans by Rows -- K = Keep Pool, C = Cached
select owner||'.'||name name, num_rows, exec, total_rows_K, K, ch from 
(select p.owner, p.name, t.num_rows,  
     sum(a.executions) exec, round(sum(a.executions)*num_rows/1000,0) total_rows_K,
     decode(t.buffer_pool,'KEEP','Y','DEFAULT','N') K,
     ltrim(t.cache) ch
  from dba_tables t,
       v$sqlarea    a,
       (select distinct address, object_owner owner, object_name name from v$sql_plan 
         where operation = 'TABLE ACCESS' and options = 'FULL') p
  where a.address = p.address
    and t.table_name = p.name
    and t.owner = p.owner
    and t.owner not in ('SYS','SYSTEM')
  group by p.owner, p.name, t.num_rows, t.cache, t.buffer_pool
  order by sum(a.executions)*num_rows desc, sum(a.executions) desc)  
where rownum<21;  
prompt
prompt Top 20 Full Table Scans by Executions -- K = Keep Pool, C = Cached
select owner||'.'||name name, num_rows, exec, K, ch from 
(select p.owner, p.name, t.num_rows, 
     sum(a.executions) exec,
     ltrim(t.cache) ch, decode(t.buffer_pool,'KEEP','Y','DEFAULT','N') K 
  from dba_tables t,
       v$sqlarea    a,
       (select distinct address, object_owner owner, object_name name from v$sql_plan 
         where operation = 'TABLE ACCESS' and options = 'FULL') p
  where a.address = p.address
    and t.table_name = p.name
    and t.owner = p.owner
    and t.owner not in ('SYS','SYSTEM')
  group by p.owner, p.name, t.num_rows, t.cache, t.buffer_pool
  order by sum(a.executions) desc, sum(a.executions) desc)  
where rownum<21;  
-- Drop the dummy table;
BEGIN
  execute immediate ('drop table v$pga_target_advice');
  execute immediate ('drop table v$sql_plan');
  EXCEPTION WHEN OTHERS THEN
    null;
END;
/

-- set heading off
-- select CHR(10)||'Disk I/O Statistics' from dual;
-- set heading on
-- set pagesize 5000
-- column name          format a50                heading "Mount"
-- column phyrds        format '99,999,999,999'   heading "Physical|Reads"
-- column phywrts       format '99,999,999,999'   heading "Physical|Writes"
-- column phyrdswrts    format '999,999,999,999'  heading "Total Physical|Reads and Writes"
-- select substr(name,1,50) name, phyrds, phywrts, phyrds+phywrts as phyrdswrts
-- from v$filestat a, v$dbfile b
-- where a.file# = b.file#
-- order by phyrds+phywrts desc;
-- clear columns

-- variable totalreadwrite number;
-- variable totalreadwritetim number;
-- BEGIN
--   select sum(phyrds+phywrts), sum(readtim+writetim) into :totalreadwrite, :totalreadwritetim
--     from v$filestat;
-- END;
-- /

-- column mount         format a30                   heading "Mount"
-- column readwrite     format '999,999,999,999'     heading "Physical|Reads and Writes"
-- column mount_pct     format '999'                 heading "Reads and Writes|as % for Mount"
-- column readwritetim  format '999,999,999,999'     heading "Time Required for|Reads and Writes"
-- column mount_tim_pct format '999'                 heading "Time Required for|Reads and Writes|as % for Mount"
-- select substr(name,1,greatest(instr(name,'\'),instr(name,'/',1,3))-1) as mount,
--   sum(phyrds+phywrts) as readwrite, 
--   round(sum(phyrds+phywrts)/:totalreadwrite*100,0) as mount_pct,
--   sum(readtim+writetim) as readwritetim,
--   round(sum(readtim+writetim)/decode(:totalreadwritetim,0,1,:totalreadwritetim)*100,0) as mount_tim_pct
-- from v$filestat a, v$dbfile b
-- where a.file# = b.file#
-- group by substr(name,1,greatest(instr(name,'\'),instr(name,'/',1,3))-1);
-- clear columns

/* DISABLED - Using too much space in USERS tablespace 
prompt
prompt Top 20 SQL Statements Causing Disk Reads
prompt ==================================================================================================
create table temp_db_checkup_topsql tablespace users as select * from 
(
 select address, disk_reads, executions, round(disk_reads/executions,0) reads_per_exec, sql_text 
 from v$sqlarea where disk_reads > 0 and executions >0 order by disk_reads desc
) where rownum < 21;
create table temp_db_checkup_topsql_plan (address raw(8), num number, plan varchar2(200)) tablespace users;
set serveroutput on size 1000000;
DECLARE
  l_size_mg number;
BEGIN
  FOR topsql_rec IN (select address from temp_db_checkup_topsql) LOOP
    -- Check to Prevent Overgrowth of temp_db_checkup_topsql_plan 
    select bytes/1024/1024 into l_size_mg from dba_segments where segment_name='TEMP_DB_CHECKUP_TOPSQL_PLAN';
    IF l_size_mg > 50 THEN
      dbms_output.put_line('Note: temp_db_checkup_topsql_plan exceded 50mg');
    ELSE
      insert into temp_db_checkup_topsql_plan (address, num, plan)
      select topsql_rec.address, rownum, 
        lpad(id,3,' ')||lpad(decode(parent_id,null,' ',parent_id),4,' ')||' '||
        rpad(lpad (' ', level - 1)||operation||' '||options,36,' ')||
        rpad(decode(object_name,null,' ',object_name),31,' ')||
        lpad(decode(cost,null,' ',cost),7,' ')||
        lpad(round(bytes/1024,0),9,' ')||
        lpad(decode(cardinality,null,' ',cardinality),9,' ') plan
      from (select address, id, parent_id, operation, options, object_name, cost, bytes, cardinality    
            from   v$sql_plan
            where  address =  topsql_rec.address
              and  rownum < 100 )
      start with id = 0
      connect by prior id = parent_id;
    END IF;
  END LOOP;
END;
/
DECLARE
  cnt number := 1;
BEGIN
  FOR sql_rec IN (select * from temp_db_checkup_topsql) LOOP
    dbms_output.put_line('Disk Reads: '||sql_rec.disk_reads||'  Executions: '||sql_rec.executions||'  Reads/Exe: '||sql_rec.reads_per_exec||'  Address: '||sql_rec.address);
    cnt := 1;
    WHILE cnt <= LENGTH(sql_rec.sql_text) LOOP
      dbms_output.put_line(SUBSTR(sql_rec.sql_text, cnt, 95));
      cnt := cnt + 95;
    END LOOP;
    dbms_output.put_line('Par/Cld Operation                         Object Name                       Cost       Kb     Card');
    dbms_output.put_line('----------------------------------------  ----------------------------- -------- --------  -------');
    FOR plan_rec IN (select * from temp_db_checkup_topsql_plan 
                     where address = sql_rec.address and num < 50 order by num) LOOP
      dbms_output.put_line(plan_rec.plan);  
    END LOOP;
    dbms_output.put_line('==================================================================================================');
  END LOOP;
END;
/
*/ 

spool off

drop table temp_db_checkup;
drop table temp_db_checkup_warn;
--drop table temp_db_checkup_topsql;
--drop table temp_db_checkup_topsql_plan;

set feedback on

